#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "getfdt.h"

#include <libfdt.h>

void fdt_close(void *fdtbuf, int fd, int size)
{
	munmap(fdtbuf, size);
	close(fd);
}

void *fdt_open(int fd, int size)
{	
	void *fdtbuf;

	fd = open("/dev/fdtexport", O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "Can't open /dev/fdtexport: %s\n",
				strerror(errno));
		return NULL;
	}

	size = lseek(fd, 0, SEEK_END);
	if (size < 0) {
		fprintf(stderr, "Can't get size to mmap: %s\n",
				strerror(errno));
		close(fd);
		return NULL;
	}

	fdtbuf = mmap(0, size, PROT_READ, MAP_SHARED, fd, 0);
	if (fdtbuf == MAP_FAILED) {
		fprintf(stderr, "Can't mmap: %s\n", strerror(errno));
		close(fd);
		return NULL;
	}

	return fdtbuf;
}


unsigned long fdt_get_ulong(const char *vpath, const char *pname,
		unsigned long defvalue)
{
	int fd, size, node;
	void *fdtbuf;
	const void *prop;
	unsigned long retval = defvalue;

	if ((fdtbuf = fdt_open(fd,size)) == NULL){
		fprintf(stderr, "Opening fdt failed!\n");
		return retval;
	}

	node = fdt_path_offset(fdtbuf, vpath);
	if (node < 0) {
		fprintf(stderr, "Can't find node '%s'\n", vpath);
		fdt_close(fdtbuf,fd,size);
		return retval;
	}

	prop = fdt_getprop(fdtbuf, node, pname, NULL);
	if (prop == NULL) {
		fprintf(stderr, "Can't find property '%s'\n", pname);
		fdt_close(fdtbuf,fd,size);
		return retval;
	}
	
	retval = fdt32_to_cpu(*((unsigned long int*)prop));
	fdt_close(fdtbuf,fd,size);
	return retval;
}


const char *fdt_get_string(const char *vpath, const char *pname,
		const char *defvalue)
{

	int fd, size, node;
	void *fdtbuf;
	const char *retval;

	if ((fdtbuf = fdt_open(fd,size)) == NULL){
		fprintf(stderr, "Opening fdt failed!\n");
		return defvalue;
	}
	
	node = fdt_path_offset(fdtbuf, vpath);
	if (node < 0) {
		fprintf(stderr, "Can't find node '%s'\n", vpath);
		fdt_close(fdtbuf,fd,size);
		return defvalue;
	}

	retval = (const char*)fdt_getprop(fdtbuf, node, pname, NULL);
	if (retval == NULL) {
		fprintf(stderr, "Can't find property '%s'\n", pname);
		fdt_close(fdtbuf,fd,size);
		return defvalue;
	}

	fdt_close(fdtbuf,fd,size);
	return retval;
}
